import { JSDOM } from 'jsdom';
import axios from 'axios';
import puppeteer from 'puppeteer';
import get from 'lodash.get';
import xpath, { XPathResult } from 'xpath-ts';

var EngineType;
(function (EngineType) {
    EngineType["Html"] = "html";
    EngineType["Json"] = "json";
    EngineType["Xml"] = "xml";
})(EngineType || (EngineType = {}));

const request$1 = async (url) => {
    try {
        const response = await axios.get(url);
        return response.data;
    }
    catch (error) {
        throw new Error(`Error fetching data from ${url}: ${error.message}`);
    }
};

const autoScroll = async (page) => {
    await page.evaluate(async () => {
        await new Promise((resolve) => {
            let totalHeight = 0;
            let distance = 100;
            let timer = setInterval(() => {
                let scrollHeight = document.body.scrollHeight;
                window.scrollBy(0, distance);
                totalHeight += distance;
                if (totalHeight >= scrollHeight) {
                    clearInterval(timer);
                    resolve(true);
                }
            }, 100);
        });
    });
};
const request = async (url, config) => {
    const browser = await puppeteer.launch({
        executablePath: config?.executablePath
            ? config.executablePath
            : puppeteer.executablePath(),
        ...config,
    });
    const page = await browser.newPage();
    await page.goto(url);
    await autoScroll(page);
    const content = await page.content();
    await browser.close();
    return content;
};

const extractData$2 = (document, selectors) => {
    const data = {};
    for (const [key, value] of Object.entries(selectors)) {
        const elements = document.querySelectorAll(value.selector);
        if (!elements.length) {
            data[key] = "";
        }
        else if (elements.length === 1) {
            if (value?.attribute) {
                data[key] = elements[0].getAttribute(value?.attribute) || "";
            }
            else {
                data[key] = elements[0].textContent?.trim() || "";
            }
        }
        else if (elements.length > 1) {
            const values = Array.from(elements, (element) => {
                if (value?.attribute) {
                    return element.getAttribute(value?.attribute) || "";
                }
                return element.textContent?.trim() || "";
            });
            data[key] = values;
        }
    }
    return data;
};
const processData$2 = async (url, options) => {
    const { selectors, spa } = options;
    if (spa?.enable) {
        const htmlContent = await request(url, spa.browserConfig);
        const dom = new JSDOM(htmlContent);
        const document = dom.window.document;
        return extractData$2(document, selectors);
    }
    else {
        const html = await request$1(url);
        const dom = new JSDOM(html);
        const document = dom.window.document;
        return extractData$2(document, selectors);
    }
};
const scrapeHtml = async (url, { selectors, spa = { enable: false, browserConfig: {} } }) => {
    const data = await processData$2(url, { selectors, spa });
    return data;
};

const extractData$1 = (j, selectors) => {
    const data = {};
    if (Array.isArray(j)) {
        return j.map((obj) => {
            const data = {};
            for (const [key, value] of Object.entries(selectors)) {
                data[key] = get(obj, value.selector, "");
            }
            return data;
        });
    }
    for (const [key, value] of Object.entries(selectors)) {
        data[key] = get(j, value.selector, "");
    }
    return data;
};
const processData$1 = (j, { selectors }) => {
    return extractData$1(j, selectors);
};
const scrapeJson = async (url, { selectors }) => {
    const j = await request$1(url);
    return processData$1(j, { selectors });
};

const extractData = (document, selectors) => {
    const data = {};
    for (const [key, value] of Object.entries(selectors)) {
        const nodes = xpath.evaluate(value.selector, document, null, XPathResult.ANY_TYPE, null);
        const nodeValues = [];
        let node = nodes.iterateNext();
        while (node) {
            nodeValues.push(node.textContent || "");
            node = nodes.iterateNext();
        }
        data[key] = nodeValues.length === 1 ? nodeValues[0] : nodeValues;
    }
    return data;
};
const processData = async (xml, { selectors }) => {
    const dom = new JSDOM(xml, { contentType: "text/xml" });
    const document = dom.window.document;
    return extractData(document, selectors);
};
const scrapeXml = async (url, { selectors }) => {
    const xml = await request$1(url);
    const data = await processData(xml, { selectors });
    return data;
};

const SCRAPE_TYPES = {
    html: scrapeHtml,
    json: scrapeJson,
    xml: scrapeXml,
};
const scrape = async ({ url, engine, options, }) => {
    return await SCRAPE_TYPES[engine](url, options);
};

export { EngineType, scrape, scrapeHtml, scrapeJson, scrapeXml };
//# sourceMappingURL=index.js.map
